package ai.search.lightout;
import java.util.Vector;

public class LightNode extends SearchNode implements Comparable{

	private boolean [][] lights;
	private  boolean [] [] hasBeenChanged ;
	private int n;
    int onCount ;


	public LightNode(int n , LightNode parent) {
		super(parent);
		this.n = n;


		lights = new boolean[n][n];

		if (parent != null) {
        	onCount = parent.onCount ;
			for(int i = 0; i < n; i++)
			{
				for(int j = 0; j < n; j++)
					lights[i][j] = parent.lights[i][j];
			}
			hasBeenChanged = parent.hasBeenChanged ;
		}
        else {
			hasBeenChanged = new boolean[n][n] ;
			for(int i = 0; i < n; i++)
			{
				for(int j = 0; j < n; j++) {
					lights[i][j] = false;
					hasBeenChanged[i][j]  = false ;
				}

			}
			onCount = 0 ;

		}
}
	
	public void makeRandomMove( int iter) {
		if (iter <= 0)
			return;

		for(int i = 0; i < iter; i++) {
			int x = (int)(Math.random() * (n - 1));
			int y = (int)(Math.random() * (n - 1));
			
			toggle(x, y);
		}
	}

	public boolean isGoal() {
		for(int i = 0; i < n; i++)
		{
			for(int j = 0; j < n; j++)
			{
				if(lights[i][j]) {
					return false;
				}
			}
		}
		
		onCount = 0 ;
		return true;
	}

	public Vector expand() {
		Vector retval = new Vector();
		if (depth + onCount / 5 > n * n ){
			//System.out.println("Bounding");
		    //System.gc() ;
			return retval ;
		}
		for( int i = 0; i < n; i++) {
			for( int j = 0; j < n; j++) {
				LightNode child = new LightNode(n , this);
				
				child.toggle( i, j);
				if (!isStupid(child.onCount , i , j ))
					retval.addElement( child);

			}
		}
		
		return retval;
	}

    public Object clone()  {
	    LightNode cloned = (LightNode)super.clone();
	    cloned.lights = new boolean [n][n];

	    for(int i = 0; i < n; i++) {
			for(int j = 0; j < n; j++)
				cloned.lights[i][j] = lights[i][j];
		}
	    
	    return cloned;
    }
    
    private void toggleLeft(int i, int j){
    	if( j - 1 < 0 | j >= n | i < 0 | i >= n)
    		return;
    	
    	if( lights[i][j - 1]) {
    		lights[i][j - 1] = false;
	        onCount-- ;
	    }
    	else {
    		lights[i][j - 1] = true;
		    onCount++ ;
	    }
    }
	
    private void toggleRight(int i, int j){    	
    	if( j < 0 | j + 1 >= n | i < 0 | i >= n)
    		return;
    	
    	if( lights[i][j + 1])   {
    		lights[i][j + 1] = false;
	        onCount-- ;
	    }
    	else{
    		lights[i][j + 1] = true;
		    onCount++ ;
	    }
    }
	
    private void toggleUp(int i, int j){    	
    	if(  j < 0 | j >= n | i - 1 < 0 | i >= n)
    		return;
    	
    	if( lights[i - 1][j]) {
    		lights[i - 1][j] = false;
	        onCount-- ;
	    }
    	else {
    		lights[i - 1][j] = true;
		    onCount++ ;
	    }
    }
	
    private void toggleDown(int i, int j){    	
    	if( j < 0 | j >= n | i < 0 | i + 1 >= n)
    		return;
    	
    	if( lights[i + 1][j]) {
    		lights[i + 1][j] = false;
	        onCount-- ;
	    }
    	else  {
    		lights[i + 1][j] = true;
	        onCount++ ;
	    }
    }
	
    private void toggle(int i, int j){    	
    	if( j < 0 | j >= n | i < 0 | i >= n)
    		return;
    	
    	if( lights[i][j])  {
    		lights[i][j] = false;
		    onCount-- ;
	    }
    	else {
    		lights[i][j] = true;
		    onCount++ ;
	    }

    	toggleLeft(i, j);
    	toggleRight(i, j);
    	toggleUp(i, j);
    	toggleDown(i, j);
    	
    	evaluate();
    }
    
    public boolean equals(Object o) {
    	if(!(o instanceof LightNode))
    		return false;
    	
    	LightNode temp = (LightNode)o;
    	
    	if( n != temp.n)
    		return false;
    	
		for(int i = 0; i < n; i++)
		{
			for(int j = 0; j < n; j++)
			{
				if(lights[i][j] != temp.lights[i][j])
					return false;
			}
		}
    	
		return true;
    }
    
    protected void evaluate() {
    }

	public int getWeight() {
		return depth + (onCount / 4) ;
	}

    public String toString() {
    	String retval = new String();

    	retval += "---";
    	
	    for(int i = 0; i < n; i++) 
	    	retval += "--";
	    
	    retval += '\n';
    	
	    for(int i = 0; i < n; i++) {
	    	retval += '\\';
			
	    	for(int j = 0; j < n; j++) {
	    		retval += '|';
	    		
	    		if(lights[i][j])
	    			retval += '1';
	    		else
	    			retval += '0';
			}
			
			retval += "|/\n";
		}
    	
    	retval += "---";
    	
	    for(int i = 0; i < n; i++) 
	    	retval += "--";
	    
	    retval += '\n';
    	
    	return retval;
    }

	public int compareTo(Object o) {
		LightNode l = (LightNode) o ;
		if (l.onCount > onCount)
			return -1 ;
		if (l.onCount < onCount)
			return 1 ;
		return 0 ;
	}

	public boolean isStupid(int count , int i , int j ) {
		int x = 5 ;
		if (i == n-1 || i== 0)
			x -= 1 ;
		if (j == n-1 || j== 0)
			x -= 1 ;

		if (count >= x + onCount)
			return true ;
		return false ;
	}
}
